package SOCC::nagios_logwatchd::Host;

use 5.006001;
use strict;
use warnings;
use English;

use fields qw(_pid _fh _config _buf _file_states);
use Data::Dumper;
use File::Temp qw(tempfile);
use POSIX qw(SIGTERM BUFSIZ);
use Fcntl qw(F_GETFL F_SETFL O_NONBLOCK);;
use Symbol qw(geniosym);

use SOCC::nagios_logwatch::Tail::FileState;



sub new {
	my ($class, $config, $file_states) = @_;
	
	my $self = fields::new($class);
	$self->{_config} = $config;
	$self->{_buf} = '';
	
	my @file_states;
	foreach my $log_file (@{$config->logs}) {
		my $file_state;
		if (defined $file_states && $file_states->{$log_file->path}) {
			$file_state = SOCC::nagios_logwatch::Tail::FileState->new_string(
				$file_states->{$log_file->path});
		} else {
			$file_state =  SOCC::nagios_logwatch::Tail::FileState->new_path(
				$log_file->path);
		}
		push(@file_states, $file_state);
	}
	$self->{_file_states} = \@file_states;
	
	return $self;
}



sub get_host_name {
	my ($self) = @_;
	
	return $self->{_config}->host_name;
}



sub get_file_states {
	my ($self) = @_;
	my %file_states;
	
	for my $file_state (@{$self->{_file_states}}) {
		$file_states{$file_state->path} = $file_state->to_string;
	}
	return \%file_states;
}



sub proc_start {
	my ($self) = @_;
	my @service_results;
	
	if ($self->{_pid}) {
		$self->proc_stop;
	}
	
	my ($fh, $conf_file) = tempfile('nagios-logwatchdXXXXXXXX',
		DIR => '/tmp');
	for my $file_state (@{$self->{_file_states}}) {
		print($fh $file_state->to_string, "\n");
	}
	close $fh;
	
	my $proc_args;
	if ($self->{_config}->isa('SOCC::nagios_logwatchd::Config::LocalHost')) {
		$proc_args = "nagios-logwatch < $conf_file";
	} elsif ($self->{_config}->isa('SOCC::nagios_logwatchd::Config::SshHost')) {
		$proc_args = "ssh " . $self->{_config}->ip_address .
			" nagios-logwatch < $conf_file";
	} else {
		die "unexpected error";
	}
	
	my $pipe = geniosym;
	my $pid = open ($pipe, "exec $proc_args |");
	if ($pid) {
		# Setup for nonblocking reads on pipe fh.
		my $flags = 0;
		fcntl($pipe, F_GETFL, $flags);
		fcntl($pipe, F_SETFL, $flags | O_NONBLOCK);
		
		$self->{_fh} = $pipe;
		$self->{_pid} = $pid;
	}
	
	
	#Call event handler
	for my $log_file (@{$self->{_config}->logs}) {
		if ($log_file->event_handler->start) {
			eval {
				my @rtn = &{$log_file->event_handler->start}(
					$self->{_config}->host_name,
					$log_file->path);
				_check_all_service_results(\@rtn);
				push(@service_results, @rtn);
			};
			if ($EVAL_ERROR ne '') {
				die ($EVAL_ERROR,
					"Event handler (start) error for ", 
					" host=", $self->{_config}->host_name,
					" path=", $log_file->path, ".\n");
			}
		}
	}
	
	return \@service_results;
}



sub proc_stop {
	my ($self) = @_;
	my @service_results;
	
	if ($self->{_pid}) {
		kill(SIGTERM, $self->{_pid});
		close $self->{_fh};
		undef $self->{_fh};
		undef $self->{_pid};
		$self->{_buf} = '';
	}
	
	for my $log_file (@{$self->{_config}->logs}) {
		if ($log_file->event_handler->stop) {
			eval {
				my @rtn = &{$log_file->event_handler->stop}(
					$self->{_config}->host_name,
					$log_file->path);
				_check_all_service_results(\@rtn);
				push(@service_results, @rtn);
			};
			if ($EVAL_ERROR ne '') {
				die ($EVAL_ERROR,
					"Event handler (stop) error for ", 
					" host=", $self->{_config}->host_name,
					" path=", $log_file->path, ".\n");
			}
		}
	}
	
	return \@service_results;
}



sub proc_fh {
	my ($self) = @_;
	
	return $self->{_fh};
}



sub read_line {
	my ($self) = @_;
	my $service_results = $self->_process_buf;;
	
	if (@{$service_results} == 0 &&
		defined $self->{_fh} && defined $self->{_fh}->fileno) {
		# This should not block.
		sysread($self->{_fh}, $self->{_buf}, BUFSIZ, length $self->{_buf});
		
		$service_results = $self->_process_buf;
	}
	
	return $service_results;
}



sub _process_buf {
	my ($self) = @_;
	my @service_results;
	
	if ($self->{_buf} =~ s/^ ([^\n]*) \n (.*) /$2/x) {
		my $input = $1;
		
		if ( $input !~ m/^ (\d+) \0 #index
			([^\0]+) \0             #state object string
			(.*)                    #line
			/x ) {
			die("Invalid input for ", $self->{_host_name}, ":$input\n");
		}
		my $idx = $1;
		my $state_obj_string = $2;
		my $line = $3;
		
		if ( $idx < 0 || $idx >= @{$self->{_config}->logs} ) {
			die ("Invalid index for", 
				" host=", $self->{_host_name},
				":$input\n"); 
		}
		
		my $state_file_obj =
			SOCC::nagios_logwatch::Tail::FileState->new_string(
				$state_obj_string);
		if ( ! defined $state_file_obj ||
			$self->{_config}->logs->[$idx]->path ne $state_file_obj->path) {
			die("Invalid state object for",
				" host=", $self->{_host_name},
				":$input\n");
		}
		$self->{_file_states}->[$idx] = $state_file_obj;
		
		#Call event handler
		eval {
			@service_results =
				&{$self->{_config}->logs->[$idx]->event_handler->line}(
					$self->{_config}->host_name, $state_file_obj->path,
					$state_file_obj->line_number, $line);
			_check_all_service_results(\@service_results);
		};
		if ($EVAL_ERROR ne '') {
			die($EVAL_ERROR,
				"Event handler (line) error for", 
				" host=", $self->{_config}->host_name,
				" path=", $state_file_obj->path,
				" line_number=", $state_file_obj->line_number,
				" line='$line'\n");
		}
	}
	
	return \@service_results;
}



sub _check_all_service_results {
	my ($service_results) = @_;
	
	foreach my $service_result (@$service_results) {
		if (ref $service_result ne 'SOCC::ServiceResult') {
			die("Return values are not all service results\n");
		}
	}
}



sub DESTROY {
	my ($self) = @_;
	
	$self->proc_stop;
}

#--------------------------------------------------------------------------

1;

